@@ -154,8 +154,13 @@ class Server(object):
                        (Python 3) are sent as text, and str (Python 2) and
                        bytes (Python 3) are sent as binary.
         """
-        self._get_socket(sid).send(packet.Packet(packet.MESSAGE, data=data,
-                                                 binary=binary))
+        try:
+            socket = self._get_socket(sid)
+        except KeyError:
+            # the socket is not available
+            self.logger.warning('Cannot send to sid %s', sid)
+            return
+        socket.send(packet.Packet(packet.MESSAGE, data=data, binary=binary))
 
     def disconnect(self, sid=None):
         """Disconnect a client.
@@ -229,8 +234,11 @@ class Server(object):
                             else:
                                 r = packets
                         except IOError:
-                            del self.sockets[sid]
+                            if sid in self.sockets:  # pragma: no cover
+                                del self.sockets[sid]
                             r = self._bad_request()
+                        if sid in self.sockets and self.sockets[sid].closed:
+                            del self.sockets[sid]
             elif method == 'POST':
                 if sid is None or sid not in self.sockets:
                     self.logger.warning('Invalid session %s', sid)
